/*
 * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef __COLLECTION_H
#define __COLLECTION_H

#include <map>
#include <vector>

using namespace std;

/** Collection
 *  Is an indexed set.
 */ 
template <class T> class Collection {
   map <T, int> index;
   vector <const T*> element;
public:
   /** Returns the collection's size. 
    */
   int size (void) { return element.size(); }

   /** Checks whether the collection has or not the element received as
    *  a parameter.  
    *  @param t element @return true if t is not in the
    *  collection
    */
   bool has_not (const T& t) { return index.find(t)==index.end(); }

   /** @param n position in the collection
    *  @return the element at the n-th position
    */
   const T& operator[] (int n) { return  *element[n]; }

   /** If the element received as a parameter does not appear in the
    *  collection, it is added at the end.  
    *  @param t an element @return
    *  the position in which t appears in the collection.
    */
   int& operator[] (const T& t) {
      if (has_not(t)) {
         index[t] = index.size()-1;
         element.push_back(&(index.find(t)->first));
      }
      return index[t];
   }

   /** Adds an element to the collection
    *  @param t the element to be added
    */  
   int& add(const T& t) {
      index[t] = index.size()-1;
      element.push_back(&(index.find(t)->first));  
      return index[t];
   }
};


#endif
