#include <apertium/TSXReader.H>
#include <lttoolbox/XMLParseUtil.H>

#include <cstdlib>
#include <iostream>

void
TSXReader::copy(TSXReader const &o)
{
}

void
TSXReader::destroy()
{
}

TSXReader::TSXReader()
{
}

TSXReader::~TSXReader()
{
  destroy();
}

TSXReader::TSXReader(TSXReader const &o)
{
  copy(o);
}

void
TSXReader::step()
{
  int retval = xmlTextReaderRead(reader);
  if(retval != 1)
  {
    parseError("unexpected EOF");
  }
  name = latin1(xmlTextReaderConstName(reader));
  type = xmlTextReaderNodeType(reader);
}

TSXReader &
TSXReader::operator =(TSXReader const &o)
{
  if(this != &o)
  {
    destroy();
    copy(o);
  }
  return *this;
}

string
TSXReader::attrib(string const &name)
{
  return XMLParseUtil::attrib(reader, name);
} 

string
TSXReader::latin1(xmlChar const * input)
{
  return XMLParseUtil::latin1(input);
}

void
TSXReader::parseError(string const &message)
{
  cerr << "Error: (" << xmlTextReaderGetParserLineNumber(reader);
  cerr << "): " << message << "." << endl;
  exit(EXIT_FAILURE);
}

void
TSXReader::newTagIndex(string const &tag)
{
  if(tag_index.find("TAG_" + tag) != tag_index.end())
  {
    parseError("'" + tag + "' already defined");
  }

  array_tags.push_back(tag);
  tag_index["TAG_" + tag] = array_tags.size() - 1;
}

void
TSXReader::procDefLabel()
{
  string name_attr = attrib("name");
  string closed_attr = attrib("closed");
  newTagIndex(name_attr);

  if(closed_attr != "true")
  {
    open_class.insert(tag_index["TAG_"+name_attr]);
  }

  while(type != XML_READER_TYPE_END_ELEMENT || name != "def-label")
  {
    step();

    if(name == "tags-item")
    {
      if(type != XML_READER_TYPE_END_ELEMENT)
      {
	int initial = recognizer[name_attr].getInitial();
	recognizer[name_attr].insertLastWord(attrib("lemma"), attrib("tags"), 
					     initial, tag_index["TAG_" + name_attr]);
      }
    }
    else if(name == "def-label")
    {
      return;
    }
    else if(name == "#text")
    {
      // do nothing
    }
    else
    {
      parseError("unexpected '<" + name + ">' tag");
    }
  }
}

void
TSXReader::procDefMult()
{
  string name_attr = attrib("name");
  string closed_attr = attrib("closed");
  newTagIndex(name_attr);
  if(closed_attr != "true")
  {
    open_class.insert(tag_index["TAG_"+name_attr]);
  }

  while(type != XML_READER_TYPE_END_ELEMENT || name != "def-mult")
  {
    step();
    cout << "DM: " << name << endl;
    if(name == "sequence")
    {
      if(type != XML_READER_TYPE_END_ELEMENT)
      {
	int current = recognizer[name_attr].getInitial();
	while(type != XML_READER_TYPE_END_ELEMENT || name != "sequence")
	{
	  step();
	  cout << "DM: " << name << endl;
	  if(name == "label-item")
	  {
	    if(type != XML_READER_TYPE_END_ELEMENT)
	    {
/*
  current = recognizer[name_attr].insert(recognizer[attr("label")], current);
*/
	    }
	  }
	  else if(name == "tags-item")
	  {
	    if(type != XML_READER_TYPE_END_ELEMENT)
	    {
/*
  current = recognizer[name_attr].insertWord(attrib("lemma"), attrib("tags"), current);
*/
	    }
	  }
	  else if(name == "sequence")
	  {
	    break;
	  }
	  else if(name == "#text")
	  {
	    // do nothing
	  }
	}
/*
  recognizer[name_attr].insertLastWord("", "", current, tag_index["TAG_" + name_attr]);
*/
      }
    }
    else if(name == "#text")
    {
      // do nothing
    }
    else if(name == "def-mult")
    { 
      // do nothing
    }
    else
    {
      parseError("unexpected '<" + name + ">' tag");
    }
  }
  cout << "SALGO" << endl;
}

void
TSXReader::procTagset()
{ 
  while(type == XML_READER_TYPE_END_ELEMENT || name != "tagset")
  {
    step();
    cout << "T: " << name << endl;
    if(name != "#text" && name != "tagger" && name != "tagset")
    {
      parseError("'<" + name + ">' tag unexpected");
    }
  }
  
  while(type != XML_READER_TYPE_END_ELEMENT || name != "tagset")
  {
    step();
    cout << "T: " << name << endl;
    if(name == "def-label")
    {
      if(type != XML_READER_TYPE_END_ELEMENT)
      {
	procDefLabel();
      }
    }
    else if(name == "def-mult")
    {
      if(type != XML_READER_TYPE_END_ELEMENT)
      {
        procDefMult();
      }
    }
    else if(name == "#text")
    {
      // do nothing
    }
    else if(name == "tagset")
    {
      // do nothing
    }
    else
    {
      parseError("Unexpected '<" + name + ">' tag");
    }
    cout << "T: " << name << endl;
  }
}


void
TSXReader::procLabelSequence()
{
  TForbidRule forbid_rule;

  step();
  if(name == "#text")
  {
    step();
  }
  if(name != "label-item")
  {
    parseError("<label-item> tag expected");
  }
  forbid_rule.tagi = tag_index[attrib("label")];

  step();
  if(name == "#text")
  {
    step();
  }
  if(name != "label-item")
  {
    parseError("<label-item> tag expected");
  }
  forbid_rule.tagj = tag_index[attrib("label")];
  
  forbid_rules.push_back(forbid_rule);
}

void
TSXReader::procForbid()
{
  while(type != XML_READER_TYPE_END_ELEMENT || name != "forbid")
  {
    step();
    if(name == "label-sequence")
    {
      if(type != XML_READER_TYPE_END_ELEMENT)
      {
	procLabelSequence();
      }
    }
    else if(name == "#text")
    {
      // do nothing
    }
    else
    {
      parseError("Unexpected '" + name + "' tag");
    }
  }  
}

void
TSXReader::procEnforce()
{
  int retval = xmlTextReaderRead(reader);
  if(retval != 1)
  {
    parseError("Unexpected EOF");
  }
  name = latin1(xmlTextReaderConstName(reader));
  
  TEnforceAfterRule enforce_rule;
  
}

void
TSXReader::procPreferences()
{
}

void
TSXReader::read(string const &filename)
{
  reader = xmlReaderForFile(filename.c_str(), NULL, 0);
  if(reader == NULL)
  {
    cerr << "Error: Cannot open '" << filename << "'." << endl;
    exit(EXIT_FAILURE);
  }

  open_class.clear();
  forbid_rules.clear();
  tag_index.clear();
  array_tags.clear();
  enforce_rules.clear();

  procTagset();
  
  step();
  if(name == "#text")
  {
    step();
  }
  if(name == "forbid")
  {
    procForbid();
    step();
    if(name == "#text")
    {
      step();
    }
  }
  if(name == "enforce-rules")
  {
    procEnforce();
    step();
    if(name == "#text")
    {
      step();
    }
  }
  if(name == "preferences")
  {
    procPreferences();
  }

  xmlFreeTextReader(reader);
  xmlCleanupParser();
}


int main(int argc, char *argv[])
{
  TSXReader tsxr;
  tsxr.read(argv[1]);
}

